
;===============================================================================
;+
; NAME:
;   Nstarks
;
; PURPOSE:
;   Defines the integrand
;     [n*(s) exp(i*k*s)]
;   Note that k and the parameters a2, a4 and a6 are specified using the common block
;
; PARAMETERS
;   sVal (in) - the input value of s.
;
; CATEGORY:
;   Quantum Fluids Analysis
;-
;===============================================================================
function Nstarks, s, k=k, muPtr=muPtr, _Extra=etc
  compile_opt idl2

  ;common nstars_common, k, muPtr
  if (n_elements(k) eq 0 || n_elements(muPtr) eq 0) then begin
    print, 'missing keyword parameter(s)'
    return, 0.0
  endif

  a2 = (*muPtr).a2
  a4 = (*muPtr).a4
  a6 = (*muPtr).a6

  return, s^2 * exp(-a2*s^2*0.5 + a4*s^4*0.041666667 - a6*s^6*0.0013888889) * Sinc(k*s)

end


;==============================================================================
; Jqys - the full Jia(s)
; alpha*[n0*beta + n0*beta*f + A*gama]
; In the IA, beta is 1 and the n0*beta term is not valid!
;==============================================================================
function Jqys, s, n0=n0, A=A, muPtr=muPtr, _Extra=etc
  compile_opt idl2

  if ((*muPtr).b3 eq 0.0 && (*muPtr).b6 eq 0.0) then begin
    ;; this is the IA, ie there is no FSE
    return, JqyAlpha(s,muPtr=muPtr,_Extra=etc)*( n0*FOFS(s,muPtr=muPtr,_Extra=etc) + $
      A*JqyGamma(s,muPtr=muPtr,_Extra=etc) )
  endif else begin
    return, JqyAlpha(s,muPtr=muPtr,_Extra=etc)*( n0*JqyBeta(s,muPtr=muPtr,_Extra=etc)* $
      (1 + FOFS(s,muPtr=muPtr,_Extra=etc)) + A*JqyGamma(s,muPtr=muPtr,_Extra=etc) )
  endelse

end


;==============================================================================
; The even function component of the
; complex Exponential part of Jqy(s)
;==============================================================================
function JqyAlpha, s, y=y, muPtr=muPtr, _Extra=etc
  compile_opt idl2

  return, Cos(  Y*s $
    + (*muPtr).b3*s^3/6.0 $
    - (*muPtr).b5*s^5/120.0)

end


;==============================================================================
; The even function component of the
; non-complex Exponential part of Rqy(s)
;==============================================================================
function JqyBeta, s, muPtr=muPtr, _Extra=etc
  compile_opt idl2

  return, Exp( (*muPtr).b4*s^4/24.0 $
    -(*muPtr).b6*s^6/720.0)
end


;==============================================================================
; The even function component of the
; non-complex Exponential part of Jqy(s)
;==============================================================================
function JqyGamma, s, muPtr=muPtr, _Extra=etc
  compile_opt idl2

  return, exp( - (*muPtr).a2*s^2/2.0 $
    + ((*muPtr).a4+(*muPtr).b4)*s^4/24.0 $
    - ((*muPtr).a6+(*muPtr).b6)*s^6/720.0 )

end


;==============================================================================
; s-space representation of the Final State function
;==============================================================================
function rqs, s, muPtr
  compile_opt idl2

  if (n_params() ne 2) then return, 0

  u3 = (*muPtr).b3
  u5 = (*muPtr).b5
  u6 = (*muPtr).b6
  calc = 0.0d
  arg = -u6*s^6/720
  calc = EXP(arg)*COS(u3*s^3/6-u5*s^5/120)

  return, calc
end
